<?php
/**
 * Whups mail processing/generation library.
 *
 * $Horde: whups/lib/Mail.php,v 1.10 2005/01/03 14:35:44 jan Exp $
 *
 * Copyright 2004-2005 Jason M. Felice <jfelice@cronosys.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * @author  Jason M. Felice <jfelice@cronosys.com>
 * @version $Revision: 1.10 $
 * @package Whups
 */
class Whups_Mail {

    /**
     * Parse a MIME message and create a new ticket.
     *
     * @param string $text              This is the full text of the MIME
     *                                  message.
     * @param array $info               An array of information for the new
     *                                  ticket.  This should include:
     *                                      'queue' => queue id
     *                                      'type' => type id
     *                                      'state' => state id
     *                                      'priority' => priority id
     * @param optional string $user     This will be the ticket's requestor.
     *                                  If null, we will try to deduce from
     *                                  the message's "From" field--we do
     *                                  NOT default to Auth::getAuth().
     *
     * @returns mixed ? or PEAR_Error:: instance.
     */
    function processMail($text, $info, $user = null)
    {
        global $conf, $whups;

        require_once 'Horde/MIME/Structure.php';
        $message = &MIME_Structure::parseTextMIMEMessage($text);

        if (preg_match("/^(.*?)\r?\n\r?\n/s", $text, $matches)) {
            $hdrText = $matches[1];
        } else {
            $hdrText = $text;
        }
        $headers = MIME_Structure::parseMIMEHeaders($hdrText);

        // Use the message subject as the ticket summary.
        $info['summary'] = Whups_Mail::_findSimpleHeaderValue($headers,
                                                              'Subject');

        $info['comment'] = Whups_Mail::_buildComment($message, $headers);

        if (is_null($user)) {
            $user = Whups_Mail::_findAuthUser($headers);
        }

        if (!is_null($user)) {
            if ($user != Auth::getAuth()) {
                $params = Horde::getDriverConfig('auth', $conf['auth']['driver']);
                $auth = &Auth::singleton($conf['auth']['driver'], $params);
                $auth->setAuth($user, array());
            }
            // FIXME: Use Identity to set email address.
        } else {
            $from = Whups_Mail::_findSimpleHeaderValue($headers, 'From');
            if (!empty($from)) {
                $info['user_email'] = $from;
            }
        }

        return $whups->addTicket($info, $user);
    }

    function _findSimpleHeaderValue($headers, $name)
    {
        foreach ($headers as $hdrName => $hdrVals) {
            if (strcasecmp($hdrName, $name) == 0) {
                if (is_array($hdrVals)) {
                    return implode(', ', $hdrVals);
                } else {
                    return $hdrVals;
                }
            }
        }

        return null;
    }

    function _buildComment(&$message, $headers)
    {
        // Format the message into a comment.
        $comment = "Received message:\n\n";
        foreach ($headers as $name => $vals) {
            if (!strcasecmp($name, 'Subject') || !strcasecmp($name, 'From') ||
                !strcasecmp($name, 'To') || !strcasecmp($name, 'Cc') ||
                !strcasecmp($name, 'Date')) {
                if (is_array($vals)) {
                    foreach ($vals as $val) {
                        $comment .= $name . ': ' . $val . "\n";
                    }
                } else {
                    $comment .= $name . ': ' . $vals . "\n";
                }
            }
        }

        $comment .= "\n";

        // Look for a text part.
        // FIXME: this is _extremely_ crude.
        if ($message->getType() == 'text/plain') {
            $comment .= $message->getContents();
        } elseif ($message->getType() == 'multipart/alternative') {
            foreach ($message->getParts() as $part) {
                if ($part->getType() == 'text/plain') {
                    $comment .= $part->getContents();
                    break;
                }
            }
        } else {
            $comment .= "[ Could not render body of message. ]";
        }

        $comment .= "\n";
        return $comment;
    }

    /**
     * Search the 'From' header for an email address contained in one
     * of our users' identities.
     *
     * @access private
     */
    function _findAuthUser($headers)
    {
        global $conf;

        require_once 'Horde/Identity.php';
        require_once 'Horde/MIME.php';

        $from = Whups_Mail::_findSimpleHeaderValue($headers, 'From');
        $from = MIME::bareAddress($from);

        $params = Horde::getDriverConfig('auth', $conf['auth']['driver']);
        $auth = &Auth::singleton($conf['auth']['driver'], $params);
        $users = $auth->listUsers();
        foreach ($users as $user) {
            $identity = &Identity::singleton('none', $user);
            $addrs = $identity->getAll('from_addr');
            foreach ($addrs as $addr) {
                if (strcasecmp($from, $addr) == 0) {
                    return $user;
                }
            }
        }

        return null;
    }

}
